unit uToyShop;

interface

uses
  classes                   // for TStrings & TStringList
  , uToy                    // for TToy
  ;

type

  TToyShop = class(TObject)
  private

    FFactoryList: TStringList;

  protected

  public

    constructor Create; virtual;
    destructor Destroy; override;

    function GetSet1(ToyType: string): TToy;
    function GetSet2(ToyType: string): TToy;
    function GetSet3(ToyType: string): TToy;
    procedure GetToyTypes(ToyTypes: TStrings);

  published

  end;


implementation

uses
  sysutils                        // for Exception
  , uToyFactory
  , uBricksFactory
  , uDollsFactory
  ;


{ TToyShop }

constructor TToyShop.Create;
begin
  // Create a factory for each toy type, with an associated string description
  if not Assigned(FFactoryList) then
    FFactoryList := TStringList.Create
  else
    FFactoryList.Clear;
  FFactoryList.AddObject('Bricks', TBricksFactory.Create);
  FFactoryList.AddObject('Dolls', TDollsFactory.Create);
end;

destructor TToyShop.Destroy;
var
  nFactory: integer;
begin
  // Free the factories and the string list
  if Assigned(FFactoryList) then
  begin
    for nFactory := 0 to Pred(FFactoryList.Count) do
      FFactoryList.Objects[nFactory].Free;
  end;
  FFactoryList.Free;
  inherited;
end;

function TToyShop.GetSet1(ToyType: string): TToy;
var
  nFactory: integer;
begin
  nFactory := FFactoryList.IndexOf(ToyType);
  if (nFactory > -1) then
    Result := TToyFactory(FFactoryList.Objects[nFactory]).GetSet1
  else
    raise Exception.Create(ToyType + ' is not a valid toy type');
end;

function TToyShop.GetSet2(ToyType: string): TToy;
var
  nFactory: integer;
begin
  nFactory := FFactoryList.IndexOf(ToyType);
  if (nFactory > -1) then
    Result := TToyFactory(FFactoryList.Objects[nFactory]).GetSet2
  else
    raise Exception.Create(ToyType + ' is not a valid toy type');
end;

function TToyShop.GetSet3(ToyType: string): TToy;
var
  nFactory: integer;
begin
  nFactory := FFactoryList.IndexOf(ToyType);
  if (nFactory > -1) then
    Result := TToyFactory(FFactoryList.Objects[nFactory]).GetSet3
  else
    raise Exception.Create(ToyType + ' is not a valid toy type');
end;

procedure TToyShop.GetToyTypes(ToyTypes: TStrings);
begin
  // Copy the list of toy types to the supplied string list
  ToyTypes.AddStrings(FFactoryList);
end;

end.
